pub struct ColumnSchemaRef<'a> {
pub column: &'a ColumnSchema,
pub table_name: &'a str,
}Expand description
Contextualizes a reference to a ColumnSchema with the name of the table the column is attached to.
Fields
column: &'a ColumnSchemaThe column we are referring to.
table_name: &'a strThe name of the table the column is attached to.